=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-03-06 06:06:07 +0000
+++ bzrlib/fetch.py	2006-03-06 12:11:25 +0000
@@ -146,20 +146,6 @@
         except errors.NoSuchRevision:
             raise InstallFailed([self._last_revision])
 
-    def _fetch_revision_texts(self, revs):
-        self.to_repository.revision_store.copy_multi(
-            self.from_repository.revision_store,
-            revs,
-            pb=self.pb)
-        # fixup inventory if needed:
-        # this is expensive because we have no inverse index to current ghosts.
-        # but on local disk its a few seconds and sftp push is already insane.
-        # so we just-do-it.
-        # FIXME: the generic code path should not need this, if it truely is
-        # generic.
-        reconciler = RepoReconciler(self.to_repository)
-        reconciler.reconcile()
-
     def _fetch_weave_texts(self, revs):
         file_ids = self.from_repository.fileid_involved_by_set(revs)
         count = 0
@@ -202,6 +188,66 @@
         self.pb.clear()
 
 
+class GenericRepoFetcher(RepoFetcher):
+    """This is a generic repo to repo fetcher.
+
+    This makes minimal assumptions about repo layout and contents.
+    It triggers a reconciliation after fetching to ensure integrity.
+    """
+
+    def _fetch_revision_texts(self, revs):
+        self.to_transaction = self.to_repository.get_transaction()
+        count = 0
+        total = len(revs)
+        for rev in revs:
+            self.pb.update('copying revisions', count, total)
+            try:
+                sig_text = self.from_repository.get_signature_text(rev)
+                self.to_repository._revision_store.add_revision_signature_text(
+                    rev, sig_text, self.to_transaction)
+            except errors.NoSuchRevision:
+                # not signed.
+                pass
+            self.to_repository._revision_store.add_revision(
+                self.from_repository.get_revision(rev),
+                self.to_transaction)
+            count += 1
+        self.pb.update('copying revisions', count, total)
+        # fixup inventory if needed: 
+        # this is expensive because we have no inverse index to current ghosts.
+        # but on local disk its a few seconds and sftp push is already insane.
+        # so we just-do-it.
+        # FIXME: repository should inform if this is needed.
+        reconciler = RepoReconciler(self.to_repository)
+        reconciler.reconcile()
+    
+
+class KnitRepoFetcher(RepoFetcher):
+    """This is a knit format repository specific fetcher.
+
+    This differs from the GenericRepoFetcher by not doing a 
+    reconciliation after copying, and using knit joining to
+    copy revision texts.
+    """
+
+    def _fetch_revision_texts(self, revs):
+        # may need to be a InterRevisionStore call here.
+        from_transaction = self.from_repository.get_transaction()
+        to_transaction = self.to_repository.get_transaction()
+        to_sf = self.to_repository._revision_store._get_signature_file(
+            to_transaction)
+        from_sf = self.from_repository._revision_store._get_signature_file(
+            from_transaction)
+        to_sf.join(from_sf, version_ids=revs, pb=self.pb, ignore_missing=True)
+        to_rf = self.to_repository._revision_store.get_revision_file(
+            to_transaction)
+        from_rf = self.from_repository._revision_store.get_revision_file(
+            from_transaction)
+        to_rf.join(from_rf, version_ids=revs, pb=self.pb)
+        reconciler = RepoReconciler(self.to_repository)
+        reconciler.reconcile()
+
+
 class Fetcher(object):
     """Backwards compatability glue for branch.fetch()."""
 

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-03-06 06:06:07 +0000
+++ bzrlib/knit.py	2006-03-06 12:11:25 +0000
@@ -829,13 +829,20 @@
         except AttributeError:
             return False
 
-    def join(self, pb=None, msg=None, version_ids=None):
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
         """See InterVersionedFile.join."""
         assert isinstance(self.source, KnitVersionedFile)
         assert isinstance(self.target, KnitVersionedFile)
 
         if version_ids is None:
             version_ids = self.source.versions()
+        else:
+            if not ignore_missing:
+                self.source._check_versions_present(version_ids)
+            else:
+                version_ids = set(self.source.versions()).intersection(
+                    set(version_ids))
+
         if not version_ids:
             return 0
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-06 10:17:26 +0000
+++ bzrlib/repository.py	2006-03-06 12:11:25 +0000
@@ -147,8 +147,6 @@
         # the following are part of the public API for Repository:
         self.bzrdir = a_bzrdir
         self.control_files = control_files
-        # backwards compatible until we fully transition
-        self.revision_store = _revision_store.text_store
         self._revision_store = _revision_store
         self.text_store = text_store
         # backwards compatability
@@ -557,6 +555,12 @@
         return self._revision_store.has_signature(revision_id,
                                                   self.get_transaction())
 
+    @needs_read_lock
+    def get_signature_text(self, revision_id):
+        """Return the text for a signature."""
+        return self._revision_store.get_signature_text(revision_id,
+                                                       self.get_transaction())
+
 
 class AllInOneRepository(Repository):
     """Legacy support - the repository behaviour for all-in-one branches."""
@@ -980,40 +984,6 @@
                                               control_files,
                                               prefixed=False)
 
-    def _get_revision_store(self, repo_transport, control_files):
-        """See RepositoryFormat._get_revision_store()."""
-        return self._get_text_rev_store(repo_transport,
-                                        control_files,
-                                        'revision-store',
-                                        compressed=False,
-                                        prefixed=True,
-                                        )
-
-    def open(self, a_bzrdir, _found=False, _override_transport=None):
-        """See RepositoryFormat.open().
-        
-        :param _override_transport: INTERNAL USE ONLY. Allows opening the
-                                    repository at a slightly different url
-                                    than normal. I.e. during 'upgrade'.
-        """
-        if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
-            assert format.__class__ ==  self.__class__
-        if _override_transport is not None:
-            repo_transport = _override_transport
-        else:
-            repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock')
-        text_store = self._get_text_store(repo_transport, control_files)
-        control_store = self._get_control_store(repo_transport, control_files)
-        _revision_store = self._get_revision_store(repo_transport, control_files)
-        return MetaDirRepository(_format=self,
-                                 a_bzrdir=a_bzrdir,
-                                 control_files=control_files,
-                                 _revision_store=_revision_store,
-                                 control_store=control_store,
-                                 text_store=text_store)
-
     def _upload_blank_content(self, a_bzrdir, dirs, files, utf8_files, shared):
         """Upload the initial blank content."""
         control_files = self._create_control_files(a_bzrdir)
@@ -1047,6 +1017,15 @@
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Repository format 7"
 
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        return self._get_text_rev_store(repo_transport,
+                                        control_files,
+                                        'revision-store',
+                                        compressed=False,
+                                        prefixed=True,
+                                        )
+
     def _get_text_store(self, transport, control_files):
         """See RepositoryFormat._get_text_store()."""
         return self._get_versioned_file_store('weaves',
@@ -1094,12 +1073,12 @@
         text_store = self._get_text_store(repo_transport, control_files)
         control_store = self._get_control_store(repo_transport, control_files)
         _revision_store = self._get_revision_store(repo_transport, control_files)
-        return KnitRepository(_format=self,
-                              a_bzrdir=a_bzrdir,
-                              control_files=control_files,
-                              _revision_store=_revision_store,
-                              control_store=control_store,
-                              text_store=text_store)
+        return MetaDirRepository(_format=self,
+                                 a_bzrdir=a_bzrdir,
+                                 control_files=control_files,
+                                 _revision_store=_revision_store,
+                                 control_store=control_store,
+                                 text_store=text_store)
 
 
 class RepositoryFormatKnit1(MetaDirRepositoryFormat):
@@ -1119,6 +1098,16 @@
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Knit Repository Format 1"
 
+    def _get_revision_store(self, repo_transport, control_files):
+        """See RepositoryFormat._get_revision_store()."""
+        from bzrlib.store.revision.knit import KnitRevisionStore
+        versioned_file_store = VersionedFileStore(
+            repo_transport.clone('revision-store'),
+            file_mode = control_files._file_mode,
+            prefixed=False,
+            versionedfile_class=KnitVersionedFile)
+        return KnitRevisionStore(versioned_file_store)
+
     def _get_text_store(self, transport, control_files):
         """See RepositoryFormat._get_text_store()."""
         return self._get_versioned_file_store('knits',
@@ -1156,6 +1145,31 @@
             bzrlib.transactions.PassThroughTransaction())
         return self.open(a_bzrdir=a_bzrdir, _found=True)
 
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = LockableFiles(repo_transport, 'lock')
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository(_format=self,
+                              a_bzrdir=a_bzrdir,
+                              control_files=control_files,
+                              _revision_store=_revision_store,
+                              control_store=control_store,
+                              text_store=text_store)
+
 
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
@@ -1201,7 +1215,7 @@
         # grab the basis available data
         if basis is not None:
             self.target.fetch(basis, revision_id=revision_id)
-        # but dont both fetching if we have the needed data now.
+        # but dont bother fetching if we have the needed data now.
         if (revision_id not in (None, NULL_REVISION) and 
             self.target.has_revision(revision_id)):
             return
@@ -1232,13 +1246,13 @@
         Returns the copied revision count and the failed revisions in a tuple:
         (copied, failures).
         """
-        from bzrlib.fetch import RepoFetcher
+        from bzrlib.fetch import GenericRepoFetcher
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, self.target._format)
-        f = RepoFetcher(to_repository=self.target,
-                        from_repository=self.source,
-                        last_revision=revision_id,
-                        pb=pb)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
         return f.count_copied, f.failed_revisions
 
     def lock_read(self):
@@ -1340,7 +1354,7 @@
                 self.target.control_weaves.copy_multi(
                     self.source.control_weaves, ['inventory'],
                     from_transaction=self.source.get_transaction())
-                self.target.revision_store._text_store.copy_all_ids(
+                self.target._revision_store.text_store.copy_all_ids(
                     self.source._revision_store.text_store,
                     pb=pb)
             else:
@@ -1349,13 +1363,13 @@
     @needs_write_lock
     def fetch(self, revision_id=None, pb=None):
         """See InterRepository.fetch()."""
-        from bzrlib.fetch import RepoFetcher
+        from bzrlib.fetch import GenericRepoFetcher
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, self.target._format)
-        f = RepoFetcher(to_repository=self.target,
-                        from_repository=self.source,
-                        last_revision=revision_id,
-                        pb=pb)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
         return f.count_copied, f.failed_revisions
 
     @needs_read_lock
@@ -1399,7 +1413,69 @@
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
 
+class InterKnitRepo(InterRepository):
+    """Optimised code paths between Knit based repositories."""
+
+    _matching_repo_format = RepositoryFormatKnit1()
+    """Repository format for testing with."""
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Knit formats.
+        
+        We dont test for the stores being of specific types becase that
+        could lead to confusing results, and there is no need to be 
+        overly general.
+        """
+        try:
+            return (isinstance(source._format, (RepositoryFormatKnit1)) and
+                    isinstance(target._format, (RepositoryFormatKnit1)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import KnitRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, self.target._format)
+        f = KnitRepoFetcher(to_repository=self.target,
+                            from_repository=self.source,
+                            last_revision=revision_id,
+                            pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    @needs_read_lock
+    def missing_revision_ids(self, revision_id=None):
+        """See InterRepository.missing_revision_ids()."""
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            assert source_ids.pop(0) == None
+        else:
+            source_ids = self.source._all_possible_ids()
+        source_ids_set = set(source_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but dont try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target._all_possible_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        required_topo_revisions = [rev_id for rev_id in source_ids if rev_id in required_revisions]
+        if revision_id is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            return required_topo_revisions
+        else:
+            # if we just grabbed the possibly available ids, then 
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            return self.source._eliminate_revisions_not_present(required_topo_revisions)
+
 InterRepository.register_optimiser(InterWeaveRepo)
+InterRepository.register_optimiser(InterKnitRepo)
 
 
 class RepositoryTestProviderAdapter(object):

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2006-03-06 10:17:26 +0000
+++ bzrlib/store/revision/knit.py	2006-03-06 12:11:25 +0000
@@ -22,6 +22,7 @@
 
 import bzrlib
 import bzrlib.errors as errors
+from bzrlib.knit import KnitVersionedFile
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.transport import get_transport
@@ -33,8 +34,10 @@
     def create(self, url):
         """Create a revision store at url."""
         t = get_transport(url)
-        t.mkdir('revstore')
-        versioned_file_store = VersionedFileStore(t.clone('revstore'))
+        t.mkdir('revision-store')
+        versioned_file_store = VersionedFileStore(
+            t.clone('revision-store'),
+            versionedfile_class=KnitVersionedFile)
         return KnitRevisionStore(versioned_file_store)
 
     def __str__(self):
@@ -55,9 +58,15 @@
 
     def _add_revision(self, revision, revision_as_file, transaction):
         """Template method helper to store revision in this store."""
+        # FIXME: make this ghost aware at the knit level
+        rf = self.get_revision_file(transaction)
+        parents = []
+        for parent_id in revision.parent_ids:
+            if rf.has_version(parent_id):
+                parents.append(parent_id)
         self.get_revision_file(transaction).add_lines(
             revision.revision_id,
-            revision.parent_ids,
+            parents,
             revision_as_file.readlines())
 
     def add_revision_signature_text(self, revision_id, signature_text, transaction):

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2006-03-06 06:06:07 +0000
+++ bzrlib/store/versioned/__init__.py	2006-03-06 12:11:25 +0000
@@ -230,5 +230,8 @@
         if pb:
             pb.clear()
 
+    def total_size(self):
+        count, bytes =  super(VersionedFileStore, self).total_size()
+        return (count / len(self._versionedfile_class.get_suffixes())), bytes
 
 WeaveStore = VersionedFileStore

=== modified file 'bzrlib/tests/blackbox/test_re_sign.py'
--- bzrlib/tests/blackbox/test_re_sign.py	2006-02-21 15:23:00 +0000
+++ bzrlib/tests/blackbox/test_re_sign.py	2006-03-06 12:11:25 +0000
@@ -53,6 +53,12 @@
 
         return wt
 
+    def assertEqualSignature(self, repo, revision_id):
+        """Assert a signature is stored correctly in repository."""
+        self.assertEqual(
+            Testament.from_revision(repo, revision_id).as_short_text(),
+            repo.get_signature_text(revision_id))
+
     def test_resign(self):
         #Test re signing of data.
         wt = self.setup_tree()
@@ -61,14 +67,10 @@
         self.monkey_patch_gpg()
         self.run_bzr('re-sign', '-r', 'revid:A')
 
-        self.assertEqual(
-            Testament.from_revision(repo, 'A').as_short_text(),
-            repo.revision_store.get('A', 'sig').read())
+        self.assertEqualSignature(repo, 'A')
 
         self.run_bzr('re-sign', 'B')
-        self.assertEqual(
-            Testament.from_revision(repo, 'B').as_short_text(),
-            repo.revision_store.get('B', 'sig').read())
+        self.assertEqualSignature(repo, 'B')
             
     def test_resign_range(self):
         wt = self.setup_tree()
@@ -76,15 +78,9 @@
 
         self.monkey_patch_gpg()
         self.run_bzr('re-sign', '-r', '1..')
-        self.assertEqual(
-            Testament.from_revision(repo, 'A').as_short_text(),
-            repo.revision_store.get('A', 'sig').read())
-        self.assertEqual(
-            Testament.from_revision(repo, 'B').as_short_text(),
-            repo.revision_store.get('B', 'sig').read())
-        self.assertEqual(
-            Testament.from_revision(repo, 'C').as_short_text(),
-            repo.revision_store.get('C', 'sig').read())
+        self.assertEqualSignature(repo, 'A')
+        self.assertEqualSignature(repo, 'B')
+        self.assertEqualSignature(repo, 'C')
 
     def test_resign_multiple(self):
         wt = self.setup_tree()
@@ -92,14 +88,6 @@
 
         self.monkey_patch_gpg()
         self.run_bzr('re-sign', 'A', 'B', 'C')
-        self.assertEqual(
-            Testament.from_revision(repo, 'A').as_short_text(),
-            repo.revision_store.get('A', 'sig').read())
-        self.assertEqual(
-            Testament.from_revision(repo, 'B').as_short_text(),
-            repo.revision_store.get('B', 'sig').read())
-        self.assertEqual(
-            Testament.from_revision(repo, 'C').as_short_text(),
-            repo.revision_store.get('C', 'sig').read())
-
-
+        self.assertEqualSignature(repo, 'A')
+        self.assertEqualSignature(repo, 'B')
+        self.assertEqualSignature(repo, 'C')

=== modified file 'bzrlib/tests/blackbox/test_sign_my_commits.py'
--- bzrlib/tests/blackbox/test_sign_my_commits.py	2006-02-17 02:01:02 +0000
+++ bzrlib/tests/blackbox/test_sign_my_commits.py	2006-03-06 12:11:25 +0000
@@ -53,6 +53,14 @@
 
         return wt
 
+    def assertUnsigned(self, repo, revision_id):
+        """Assert that revision_id is not signed in repo."""
+        self.assertFalse(repo.has_signature_for_revision_id(revision_id))
+
+    def assertSigned(self, repo, revision_id):
+        """Assert that revision_id is signed in repo."""
+        self.assertTrue(repo.has_signature_for_revision_id(revision_id))
+
     def test_sign_my_commits(self):
         #Test re signing of data.
         wt = self.setup_tree()
@@ -60,17 +68,17 @@
 
         self.monkey_patch_gpg()
 
-        self.assertFalse(repo.revision_store.has_id('A', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('B', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('C', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('D', 'sig'))
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
 
         self.run_bzr('sign-my-commits')
 
-        self.assertTrue(repo.revision_store.has_id('A', 'sig'))
-        self.assertTrue(repo.revision_store.has_id('B', 'sig'))
-        self.assertTrue(repo.revision_store.has_id('C', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('D', 'sig'))
+        self.assertSigned(repo, 'A')
+        self.assertSigned(repo, 'B')
+        self.assertSigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
             
     def test_sign_my_commits_location(self):
         wt = self.setup_tree('other')
@@ -80,10 +88,10 @@
 
         self.run_bzr('sign-my-commits', 'other')
 
-        self.assertTrue(repo.revision_store.has_id('A', 'sig'))
-        self.assertTrue(repo.revision_store.has_id('B', 'sig'))
-        self.assertTrue(repo.revision_store.has_id('C', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('D', 'sig'))
+        self.assertSigned(repo, 'A')
+        self.assertSigned(repo, 'B')
+        self.assertSigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')
 
     def test_sign_diff_committer(self):
         wt = self.setup_tree()
@@ -93,10 +101,10 @@
 
         self.run_bzr('sign-my-commits', '.', 'Alternate <alt@foo.com>')
 
-        self.assertFalse(repo.revision_store.has_id('A', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('B', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('C', 'sig'))
-        self.assertTrue(repo.revision_store.has_id('D', 'sig'))
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertSigned(repo, 'D')
 
     def test_sign_dry_run(self):
         wt = self.setup_tree()
@@ -107,8 +115,7 @@
         out = self.run_bzr('sign-my-commits', '--dry-run')[0]
 
         self.assertEquals('A\nB\nC\nSigned 3 revisions\n', out)
-        self.assertFalse(repo.revision_store.has_id('A', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('B', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('C', 'sig'))
-        self.assertFalse(repo.revision_store.has_id('D', 'sig'))
-
+        self.assertUnsigned(repo, 'A')
+        self.assertUnsigned(repo, 'B')
+        self.assertUnsigned(repo, 'C')
+        self.assertUnsigned(repo, 'D')

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-02-21 12:42:57 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-03-06 12:11:25 +0000
@@ -242,16 +242,19 @@
         branch.repository.sign_revision('A', strategy)
         self.assertEqual(Testament.from_revision(branch.repository, 
                          'A').as_short_text(),
-                         branch.repository.revision_store.get('A', 
-                         'sig').read())
+                         branch.repository.get_signature_text('A'))
 
     def test_store_signature(self):
-        branch = self.get_branch()
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
         branch.repository.store_revision_signature(
             bzrlib.gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+        self.assertRaises(errors.NoSuchRevision,
+                          branch.repository.has_signature_for_revision_id,
+                          'A')
+        wt.commit("base", allow_pointless=True, rev_id='A')
         self.assertEqual('FOO', 
-                         branch.repository.revision_store.get('A', 
-                         'sig').read())
+                         branch.repository.get_signature_text('A'))
 
     def test_branch_keeps_signatures(self):
         wt = self.make_branch_and_tree('source')
@@ -262,10 +265,8 @@
         # wt.clone should work to disks.
         self.build_tree(['target/'])
         d2 = wt.bzrdir.clone('target')
-        self.assertEqual(wt.branch.repository.revision_store.get('A', 
-                            'sig').read(),
-                         d2.open_repository().revision_store.get('A', 
-                            'sig').read())
+        self.assertEqual(wt.branch.repository.get_signature_text('A'),
+                         d2.open_repository().get_signature_text('A'))
 
     def test_nicks(self):
         """Branch nicknames"""

=== modified file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-03-06 06:06:07 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-03-06 12:11:25 +0000
@@ -163,10 +163,10 @@
         
     def test_fetch_preserves_signatures(self):
         from_repo = self.bzrdir.open_repository()
-        from_signature = from_repo.revision_store.get('rev2', 'sig').read()
+        from_signature = from_repo.get_signature_text('rev2')
         to_repo = self.make_to_repository('target')
         to_repo.fetch(from_repo)
-        to_signature = to_repo.revision_store.get('rev2', 'sig').read()
+        to_signature = to_repo.get_signature_text('rev2')
         self.assertEqual(from_signature, to_signature)
 
 

=== modified file 'bzrlib/tests/interversionedfile_implementations/test_join.py'
--- bzrlib/tests/interversionedfile_implementations/test_join.py	2006-03-06 06:06:07 +0000
+++ bzrlib/tests/interversionedfile_implementations/test_join.py	2006-03-06 12:11:25 +0000
@@ -148,6 +148,21 @@
         eq(w1.get_lines('v2'), ['hello\n', 'world\n'])
         eq(w1.get_parents('v2'), ['v1', 'x1'])
 
+    def test_join_with_ignore_missing_versions(self):
+        # test that ignore_missing=True makes a listed but absent version id
+        # be ignored, and that unlisted version_ids are not integrated.
+        w1 = self.build_weave1()
+        wb = self.get_target()
+        wb.add_lines('x1', [], ['line from x1\n'])
+        wb.add_lines('v1', [], ['hello\n'])
+        wb.add_lines('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        w1.join(wb, version_ids=['x1', 'z1'], ignore_missing=True)
+        eq = self.assertEquals
+        eq(sorted(w1.versions()), ['v1', 'v2', 'v3', 'x1'])
+        eq(w1.get_text('x1'), 'line from x1\n')
+        eq(w1.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(w1.get_parents('v2'), ['v1'])
+    
     def build_source_weave(self, name, *pattern):
         w = self.get_source(name)
         for version, parents in pattern:

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-03-06 10:17:26 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-03-06 12:11:25 +0000
@@ -226,8 +226,7 @@
         wt.commit('A', allow_pointless=True, rev_id='A')
         wt.branch.repository.sign_revision('A',
             bzrlib.gpg.LoopbackGPGStrategy(None))
-        old_signature = wt.branch.repository.revision_store.get('A',
-            'sig').read()
+        old_signature = wt.branch.repository.get_signature_text('A')
         try:
             old_format = bzrdir.BzrDirFormat.get_default_format()
             # This gives metadir branches something they can convert to.
@@ -241,8 +240,7 @@
             # this is in the most current format already.
             return
         wt = WorkingTree.open(wt.basedir)
-        new_signature = wt.branch.repository.revision_store.get('A',
-            'sig').read()
+        new_signature = wt.branch.repository.get_signature_text('A')
         self.assertEqual(old_signature, new_signature)
 
 

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-01-27 05:28:12 +0000
+++ bzrlib/tests/test_commit.py	2006-03-06 12:11:25 +0000
@@ -308,7 +308,7 @@
         wt = self.make_branch_and_tree('.')
         branch = wt.branch
         wt.commit("base", allow_pointless=True, rev_id='A')
-        self.failIf(branch.repository.revision_store.has_id('A', 'sig'))
+        self.failIf(branch.repository.has_signature_for_revision_id('A'))
         try:
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
@@ -319,8 +319,7 @@
                                                       working_tree=wt)
             self.assertEqual(Testament.from_revision(branch.repository,
                              'B').as_short_text(),
-                             branch.repository.revision_store.get('B', 
-                                                               'sig').read())
+                             branch.repository.get_signature_text('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
 
@@ -331,7 +330,7 @@
         wt = self.make_branch_and_tree('.')
         branch = wt.branch
         wt.commit("base", allow_pointless=True, rev_id='A')
-        self.failIf(branch.repository.revision_store.has_id('A', 'sig'))
+        self.failIf(branch.repository.has_signature_for_revision_id('A'))
         try:
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
@@ -345,7 +344,7 @@
                               working_tree=wt)
             branch = Branch.open(self.get_url('.'))
             self.assertEqual(branch.revision_history(), ['A'])
-            self.failIf(branch.repository.revision_store.has_id('B'))
+            self.failIf(branch.repository.has_revision('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
 

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-03-02 11:01:15 +0000
+++ bzrlib/versionedfile.py	2006-03-06 12:11:25 +0000
@@ -170,15 +170,22 @@
     def annotate(self, version_id):
         return list(self.annotate_iter(version_id))
 
-    def join(self, other, pb=None, msg=None, version_ids=None):
+    def join(self, other, pb=None, msg=None, version_ids=None,
+             ignore_missing=False):
         """Integrate versions from other into this versioned file.
 
         If version_ids is None all versions from other should be
         incorporated into this versioned file.
 
         Must raise RevisionNotPresent if any of the specified versions
-        are not present in the other files history."""
-        return InterVersionedFile.get(other, self).join(pb, msg, version_ids)
+        are not present in the other files history unless ignore_missing
+        is supplied when they are silently skipped.
+        """
+        return InterVersionedFile.get(other, self).join(
+            pb,
+            msg,
+            version_ids,
+            ignore_missing)
 
     def walk(self, version_ids=None):
         """Walk the versioned file as a weave-like structure, for
@@ -312,14 +319,15 @@
     _optimisers = set()
     """The available optimised InterVersionedFile types."""
 
-    def join(self, pb=None, msg=None, version_ids=None):
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
         """Integrate versions from self.source into self.target.
 
         If version_ids is None all versions from source should be
         incorporated into this versioned file.
 
         Must raise RevisionNotPresent if any of the specified versions
-        are not present in the other files history.
+        are not present in the other files history unless ignore_missing is 
+        supplied when they are silently skipped.
         """
         # the default join: 
         # - make a temporary versioned file of type target
@@ -335,7 +343,11 @@
                                   self.source.get_lines(version))
         
         # this should hit the native code path for target
-        return self.target.join(temp_source, pb, msg, version_ids)
+        return self.target.join(temp_source,
+                                pb,
+                                msg,
+                                version_ids,
+                                ignore_missing)
 
 
 class InterVersionedFileTestProviderAdapter(object):

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-03-06 09:38:37 +0000
+++ bzrlib/weave.py	2006-03-06 12:11:25 +0000
@@ -730,23 +730,28 @@
         # no lines outside of insertion blocks, that deletions are
         # properly paired, etc.
 
-    def _join(self, other, pb, msg, version_ids):
+    def _join(self, other, pb, msg, version_ids, ignore_missing):
         """Worker routine for join()."""
         if not other.versions():
             return          # nothing to update, easy
 
         if version_ids:
             for version_id in version_ids:
-                if not self.has_version(version_id):
+                if not self.has_version(version_id) and not ignore_missing:
                     raise RevisionNotPresent(version_id, self._weave_name)
-        assert version_ids == None
+        else:
+            version_ids = other.versions()
 
         # two loops so that we do not change ourselves before verifying it
         # will be ok
         # work through in index order to make sure we get all dependencies
         names_to_join = []
         processed = 0
-        for other_idx, name in enumerate(other._names):
+        version_ids = set(other.versions()).intersection(set(version_ids))
+        pending_graph = [(version, other.get_parents(version)) for
+                         version in version_ids]
+        for name in topo_sort(pending_graph):
+            other_idx = other._name_map[name]
             self._check_version_consistent(other, other_idx, name)
             sha1 = other._sha1s[other_idx]
 
@@ -893,9 +898,10 @@
         """See VersionedFile.get_suffixes()."""
         return [WeaveFile.WEAVE_SUFFIX]
 
-    def join(self, other, pb=None, msg=None, version_ids=None):
+    def join(self, other, pb=None, msg=None, version_ids=None,
+             ignore_missing=False):
         """Join other into self and save."""
-        super(WeaveFile, self).join(other, pb, msg, version_ids)
+        super(WeaveFile, self).join(other, pb, msg, version_ids, ignore_missing)
         self._save()
 
 
@@ -1221,10 +1227,10 @@
         except AttributeError:
             return False
 
-    def join(self, pb=None, msg=None, version_ids=None):
+    def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
         """See InterVersionedFile.join."""
         try:
-            self.target._join(self.source, pb, msg, version_ids)
+            self.target._join(self.source, pb, msg, version_ids, ignore_missing)
         except errors.WeaveParentMismatch:
             self.target._reweave(self.source, pb, msg)
 

